/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: console.h,v 1.6 2003/09/03 11:13:45 rodzilla Exp $

    The CConsole class is a basic console that logs everything and
    defines an interface for an interpreter.
*/

#ifndef __SAM3D_CONSOLE_H__
#define __SAM3D_CONSOLE_H__

#include <string>
#include <SDL/SDL.h>
#include <SaM3d/font.h>

namespace SaM3d {

	//!	A very simple console class.
	class CConsole {
	public:
		//!	The interpreter class.
		/*!	Inherit from this class when writing an interpreter
			and it can be automagically fed code through the
			console.
		*/
		class Interpreter {
		public:
			virtual ~Interpreter() { }

			//!	Interpret a string.
			virtual void do_string(const std::string& s) = 0;
		};

	protected:
		//!	Singleton stuff.
		static CConsole* m_instance;

		//!	The font used to render the console.
		SaM3d::Font m_font;

		//!	The position in the text buffer.
		int  m_position;
		char m_mode;

		unsigned int m_text_height;

		//!	The current command line.
		std::string m_command_line;

		//!	The interpreter bound to the console.
		Interpreter*  m_interpreter;

		//!	Constructor.
		CConsole();

	public:
		//!	Destructor.
		~CConsole();

		//!	Return the unique instance of the console.
		static CConsole& instance();

		//!	Bind an interpreter to the console.
		void set_interpreter(Interpreter* interpreter);

		//!	Draw the console.
		void draw();

		//!	Handle a key.
		/*!	\return true if the key was handled, false if it was ignored.
			\param key the key symbol of the key to be handled.
		*/
		bool handle_key_pressed(const SDL_keysym* key);

		//!	Is the console active ?
		bool is_active() const;
	};

}

#define Console SaM3d::CConsole::instance()

#endif
