/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: file_texture.h,v 1.5 2003/08/28 13:48:22 rodzilla Exp $
*/

#ifndef __SAM3D_FILE_TEXTURE_H__
#define __SAM3D_FILE_TEXTURE_H__

#include <SaM3d/file.h>
#include <SaM3d/texture.h>

namespace SaM3d {

	//!	Texture created from an image file.
	class FileTexture : public BaseTexture {
		//!	Directory that contains the image file.
		Dir m_dir;

		//!	Name of the image file
		std::string m_filename;

		//!	List of accepted extensions for the image file.
		char** m_extensions;

		//!	Generate mipmaps or not ?
		bool m_mipmap;

		//!	Texture wrap mode.
		GLenum m_texture_wrap;

	protected:
		//! 	Create the texture.
		virtual void create();

		virtual unsigned is_a() const { return id_file_texture; }
		virtual bool equal(const BaseTexture* t) const;
		virtual unsigned char hash() const;

	public:
		//!	Constructor.
		/*!	\param dir The directory in which the image file must be sought.
			\param filename the name of the image file.
			\param extensions the accepeted extensions for the image file.
			\param mipmap set to true to generate mipmaps.
			\param texture_wrap texture wrap mode (GL_REPEAT, GL_CLAMP or GL_CLAMP_TO_EDGE).
		*/
		FileTexture(const Dir& dir,
			    const char* filename,
			    char** extensions = NULL,
			    bool mipmap = false,
			    GLenum texture_wrap = GL_REPEAT);
		//!	Constructor.
		/*!	When using this constructor, the image file will be sought in
			the current directory.
			\param filename the name of the image file.
			\param extensions the accepeted extensions for the image file.
			\param mipmap set to true to generate mipmaps.
			\param texture_wrap texture wrap mode (GL_REPEAT, GL_CLAMP or GL_CLAMP_TO_EDGE).
		*/
		FileTexture(const char* filename,
			    char** extensions = NULL,
			    bool mipmap = false,
			    GLenum texture_wrap = GL_REPEAT);

		//!	Destructor.
		virtual ~FileTexture();
	};

}

#endif
