/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: lightmap.h,v 1.4 2003/08/28 13:48:22 rodzilla Exp $
*/

#ifndef __SAM3D_LIGHTMAP_H__
#define __SAM3D_LIGHTMAP_H__

#include <SaM3d/config_variable.h>
#include <SaM3d/status.h>
#include <SaM3d/texture.h>

namespace SaM3d {

	//!	Class for Q3A lightmaps.
	/*!	\todo Turn this into a more generic MemTexture class.
	*/
	class LightMap : public BaseTexture {
		struct GammaHandler : public CfgVar::NumHandler {
			float m_min, m_max;

			GammaHandler(const float min, const float max);
			virtual bool operator ()(float& value) const;
		};
	friend class GammaHandler;

		//!	Handler for the gamma config variable.
		static GammaHandler M_gamma_handler;

		//!	Used to store the date of last gamma correction change.
		static Status M_gamma_status;

		//!	Reference data for the lightmap.
		unsigned char*	m_reference_data;

		//!	Used to know if the gamma for this lightmap is up-to-date.
		Status m_gamma_status;

	protected:
		//!	Create the texture.
		virtual void create();

		virtual unsigned is_a() const { return id_lightmap; }
		virtual bool equal(const BaseTexture* t) const;
		virtual unsigned char hash() const;

	public:
		//!	Create the texture.
		/*!	The texture is created using data that must contain
			and RGB description of a 128x128 image.
		*/
		LightMap(unsigned char* data);

		//!	Destructor.
		virtual ~LightMap();

		//!	The gamma correction of all lightmaps.
		static FloatCfgVar gamma;
	};

}

#endif
