/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: performance_counter.h,v 1.2 2003/08/28 13:48:22 rodzilla Exp $

*/

#ifndef __SAM3D_PERFORMANCE_COUNTER_H__
#define __SAM3D_PERFORMANCE_COUNTER_H__

#include <deque>

namespace SaM3d {

	//!	A simple performance measuring class.
	class CPerformanceCounter {
	protected:
		//!	Frames rendered during the last second.
		std::deque<int> m_last_frames;
		//!	Triangles rendered during the last frame.
		unsigned m_triangles;

		//!	Constructor.
		CPerformanceCounter();

	public:
		//!	Returns a reference to the (unique) instance.
		static CPerformanceCounter& instance();

		//!	Tell the performance counter that a frame has been rendered.
		/*!	\param t the current time.
		*/
		void add_frame(const unsigned t);

		//!	Return the average FPS during the last second.
		unsigned fps() const;

		//!	Tell the performance counter that triangles have been rendered.
		/*!	\param t the number of triangles.
		*/
		void add_triangles(const unsigned t);

		//!	Return the total number of triangles since the last frame.
		unsigned triangles() const;
	};

}

#define PerformanceCounter SaM3d::CPerformanceCounter::instance()

#endif
