/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: viewport.h,v 1.2 2003/08/28 13:48:22 rodzilla Exp $

    The Viewport class.
*/

#ifndef __SAM3D_VIEWPORT_H__
#define __SAM3D_VIEWPORT_H__

#include <SaM3d/config.h>
#include <list>

namespace SaM3d {

	//!	A viewport class.
	/*!	This is a class for OpenGL viewports.
		It's useful to abstract glViewport and to provide a current
		viewport to the Camera class (the current Viewport's aspect
		is required when activating a Camera).
	*/
	class Viewport {
		//!	The current viewport.
		static const Viewport* m_current;

		//!	This is true if we're in ortho mode.
		static bool m_view_ortho;

		//!	The position and size of the viewport.
		int m_x, m_y, m_w, m_h;

	public:
		//!	Basic constructor.
		Viewport(const int x = 0, const int y = 0,
			 const int w = 1, const int h = 1);

		//!	Get the current viewport.
		/*!	Note that this method returns a const reference.
		        This prevents resizing Display.viewport().
			\return a const reference to the current Viewport.
		*/
		static const Viewport& current();

		//!	Activate the Viewport.
		/*!	Define this as the current viewport and call glViewport.
		*/
		void activate() const;

		//!
		int x() const { return m_x; }
		//!
		int y() const { return m_y; }
		//!
		int w() const { return m_w; }
		//!
		int h() const { return m_h; }

		//!	Move and resize the Viewport.
		void move_resize(const int x, const int y, const int w, const int h);

		//!	Begin ortho view mode.
		void begin_view_ortho() const;

		//!	End ortho view mode.
		void end_view_ortho() const;

	friend class CDisplay;
	};

}

#endif
