/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: font.cpp,v 1.3 2003/08/24 15:45:00 rodzilla Exp $

    The SM::Font class handles text rendering.
    It is based on NeHe's (nehe.gamedev.net) bitmap fonts.
*/

#include <stdarg.h>
#include <stdio.h>
#include <SaM3d/font.h>

//#define SAM3D_DEBUG
#include <SaM3d/debug.h>

namespace SaM3d {

	Font::
	Font() : m_texture(new FileTexture("textures/font.tga", NULL, false)) {
		DEBUG_FUNCTION("Font::Font()");
	}

	Font::
	~Font() {
		DEBUG_FUNCTION("Font::~Font()");
	}

	void
	Font::
	print(const float x, const float y, const float fs, const char *text) {
		DEBUG_FUNCTION("Font::print(x, y, fs, text)");
		unsigned l = strlen(text);
		unsigned i;
		float tmp_x = 0.0;

		glDisable(GL_LIGHTING);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		m_texture.bind();
		glPushMatrix();
		glScalef(fs, fs, 0);
		glTranslatef(x/fs, y/fs, 0);
		glBegin(GL_QUADS);
		for (i = 0; i < l; ++i) {
			if (text[i] > 32) {
				char c = text[i]-32;
				float cx=float(c%16)*0.0625f;
				float cy=float(c>>4)*0.0625f;

				glTexCoord2f(cx,1.0f-cy-0.001f);
				glVertex2f(tmp_x,0);
				glTexCoord2f(cx,1.0f-cy-0.0625f);
				glVertex2f(tmp_x, -1.05);
				glTexCoord2f(cx+0.0625f,1.0f-cy-0.0625f);
				glVertex2f(tmp_x+1.05, -1.05);
				glTexCoord2f(cx+0.0625f,1.0f-cy-0.001f);
				glVertex2f(tmp_x+1.05,0);
			}
			tmp_x += 1.0;
		}
		glEnd();
		glPopMatrix();
	}

	void
	Font::
	printf(const float x, const float y, const float fs, const char *fmt, ...) {
		static char text[1024];
		va_list	ap;

		if (fmt == NULL) return;

		va_start(ap, fmt);
		vsprintf(text, fmt, ap);
		va_end(ap);

		print(x, y, fs, text);
	}

	void
	Font::
	printf(const float x, const float y, const char *fmt, ...) {
		static char text[1024];
		va_list	ap;

		if (fmt == NULL) return;

		va_start(ap, fmt);
		vsprintf(text, fmt, ap);
		va_end(ap);

		print(x, y, 12, text);
	}

	unsigned int
	Font::
	get_width(const char* text) {
		return 12*strlen(text);
	}

	unsigned int
	Font::
	get_height() {
		return 12;
	}

}

