/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: image_transform.cpp,v 1.2 2003/08/31 10:04:34 rodzilla Exp $
*/

#include <assert.h>
#include <string.h>
#include <SaM3d/log.h>
#include <SaM3d/texture.h>
#include "image_transform.h"

namespace SaM3d {
	namespace ImageTransform {

		using namespace std;

		GLenum force_supported_format(SDL_Surface* image) {
			assert(image != NULL);

			GLenum format = 0;

			switch (image->format->BytesPerPixel) {
				case 3:
					if (   image->format->Rshift == 16
					    && image->format->Gshift == 8) {
						format = GL_BGR;
					} else if (   image->format->Bshift == 16
					           && image->format->Gshift == 8) {
						format = GL_RGB;
					}
					break;
				case 4:
					if (   image->format->Rshift == 16
					    && image->format->Gshift == 8) {
						format = GL_BGRA;
					} else if (   image->format->Bshift == 16
					           && image->format->Gshift == 8) {
						format = GL_RGBA;
					}
					break;
				default:
					printf("Unrecognized image format\n");
					break;
			}

			if (   (!Texture::has_bgra_support())
			    && (format == GL_BGR || format == GL_BGRA)) {
				char* pixels = (char*)(image->pixels);
				unsigned int bpp = image->format->BytesPerPixel;
				unsigned int data_size = bpp*image->w*image->h;
				unsigned int i;

				for (i = 0; i < data_size; i += bpp) {
					char tmp = pixels[i];
					pixels[i] = pixels[i+2];
					pixels[i+2] = tmp;
				}

				if (format == GL_BGR) {
					format = GL_RGB;
					image->format->Bshift = 16;
					image->format->Gshift = 8;
				} else {
					format = GL_RGBA;
					image->format->Bshift = 16;
					image->format->Gshift = 8;
				}
			}

			return format;
		}

		void vertical_flip(SDL_Surface* image) {
			assert(image != NULL);

			char* buf = new char[image->pitch];
			int i1;

			// Copy the buffer into the SDL surface while flipping it vertically.
			for (i1 = 0; i1 < image->h >> 1; ++i1) {
				int i2 = image->h-i1-1;

				if (i1 != i2) {
					memcpy(buf, ((char *)image->pixels) + image->pitch*i1, image->pitch);
					memcpy(((char *)image->pixels) + image->pitch*i1,
					       ((char *)image->pixels) + image->pitch*i2, image->pitch);
					memcpy(((char *)image->pixels) + image->pitch*i2, buf, image->pitch);
				}
			}
		}

		void fix_size(SDL_Surface* image) {
			int new_w, new_h;
			void* new_pixels;
			int i, j;

			if (image == NULL) return;

			if (image->w & (image->w-1)) {
				for (new_w = 4; new_w <= image->w; new_w <<= 1);
			} else {
				new_w = image->w;
			}

			if (image->h & (image->h-1)) {
				for (new_h = 4; new_h < image->h; new_h <<= 1);
			} else {
				if (new_w == image->w) return;
				new_h = image->h;
			}

			int bpp = image->format->BytesPerPixel;

			new_pixels = malloc(new_w*new_h*bpp);

			for (i = 0; i < new_w; ++i) {
				for (j = 0; j < new_h; ++j) {
					int old_index = (i*image->w/new_w)*bpp+image->pitch*(j*image->h/new_h);
					int new_index = (i+new_w*j)*bpp;
					int k;

					for (k = 0; k < bpp; ++k) {
						((char*)new_pixels)[new_index+k] = ((char*)(image->pixels))[old_index+k];
					}
				}
			}

			image->w = new_w;
			image->h = new_h;
			image->pitch = new_w*bpp;
			free(image->pixels);
			image->pixels = new_pixels;
		}

	}

}

