/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: lightmap.cpp,v 1.3 2003/08/11 15:41:43 rodzilla Exp $
*/

#include <SaM3d/lightmap.h>
#include <SaM3d/log.h>

namespace SaM3d {

	using namespace std;

	LightMap::GammaHandler::
	GammaHandler(const float min,
		     const float max) : m_min(min), m_max(max) { }

	bool
	LightMap::GammaHandler::
	operator ()(float& value) const {
		if (value < m_min) value = m_min;
		if (value > m_max) value = m_max;
		LightMap::M_gamma_status.update();
		return true;
	}

	LightMap::GammaHandler LightMap::M_gamma_handler(0, 20);

	FloatCfgVar LightMap::gamma("lm_gamma", 2.5, M_gamma_handler);
	Status LightMap::M_gamma_status;

	LightMap::
	LightMap(unsigned char* data) : BaseTexture() {
		m_reference_data = data;
		m_status = uninitialized;
	}

	LightMap::
	~LightMap() {
		delete [] m_reference_data;
	}

	bool
	LightMap::
	equal(const BaseTexture* t) const {
		if (t != NULL && t->is_a() == id_lightmap) {
			LightMap* lm = (LightMap*)t;
			unsigned i;

			for (i = 0; i < 49152; ++i) {
				if (m_reference_data[i] != lm->m_reference_data[i]) {
					return false;
				}
			}

			return true;
		}
		return false;
	}

	unsigned char
	LightMap::
	hash() const {
		unsigned char rv = 0;
		unsigned i;

		for (i = 0; i < 49152; i += 385) {
			rv += m_reference_data[i];
		}

		return rv;
	}

	void
	LightMap::
	create() {
		if (   (m_status == uninitialized)
		    || (! m_gamma_status.is_up_to_date(M_gamma_status))) {
			unsigned i;
			static unsigned char data[49152];

			for (i = 0; i < 49152; i += 3) {
				float r, g, b;
				float scale = gamma;
				float tmp;

				r = (float)m_reference_data[i];
				g = (float)m_reference_data[i+1];
				b = (float)m_reference_data[i+2];

				if ((tmp = 255.0/r) < scale) scale = tmp;
				if ((tmp = 255.0/g) < scale) scale = tmp;
				if ((tmp = 255.0/b) < scale) scale = tmp;

				data[i] = (unsigned char)(r * scale);
				data[i+1] = (unsigned char)(g * scale);
				data[i+2] = (unsigned char)(b * scale);
			}

			if (m_status == uninitialized) {
				glGenTextures(1, &m_id);
			}
			glBindTexture(GL_TEXTURE_2D, m_id);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, 128, 128, 0, GL_RGB,
				     GL_UNSIGNED_BYTE, data);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

			m_status = ok;
			m_gamma_status.update();
		}
	}

}

