/**************************************************************************

    This file is part of "The Science and Magick base library", a
    general library that provides miscellaneous features.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: log.cpp,v 1.3 2003/08/09 09:35:15 rodzilla Exp $
*/

#include <SaM3d/config.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <SaM3d/log.h>
#include <iostream>

namespace SaM3d {

	using namespace std;

	CLog::
	CLog() : m_log_file("log.txt") {
		m_nb_lines = 1;
		m_current_line = 0;
	}

	void 
	CLog::
	add_string(char* s) {
		char* tmp;

		for (tmp=s; tmp != NULL;) {
			add_chars(strsep(&tmp, "\n"));
			if (tmp) {
				endl();
			}
		}
	}

	void 
	CLog::
	add_chars(const char* s) {
		m_lines[m_current_line] += s;
	}

	void 
	CLog::
	endl() {
		if (m_lines[m_current_line].size() > 0) {
			m_log_file << m_lines[m_current_line] << std::endl;

			if (++m_current_line == m_max_lines) {
				m_current_line = 0;
			}
			if (m_nb_lines < m_max_lines) {
				m_nb_lines++;
			}
			m_lines[m_current_line] = "";
		}
	}

	CLog&
	CLog::
	instance() {
		static CLog instance;
		return instance;
	}

	CLog&
	CLog::
	operator<<(const bool b) {
		if (b) {
			add_chars("true");
		} else {
			add_chars("false");
		}
		return *this;
	}

	CLog&
	CLog::
	operator<<(const int i) {
		char buffer[32];

		snprintf(buffer, 31, "%d", i);
		add_chars(buffer);
		return *this;
	}

	CLog&
	CLog::
	operator<<(const float f) {
		return operator<<((double)f);
	}

	CLog&
	CLog::
	operator<<(const double d) {
		char buffer[32];

		snprintf(buffer, 31, "%e", d);
		add_chars(buffer);
		return *this;
	}

	CLog&
	CLog::
	operator<<(const char* s) {
		char* tmp = strdup(s);
		add_string(tmp);
		free(tmp);
		return (*this);
	}

	CLog&
	CLog::
	operator<<(const string& s) {
		return operator<<(s.c_str());
	}

	void 
	CLog::
	printf(const char* fmt, ...) {
		va_list arg_list;

		va_start(arg_list, fmt);
		vprintf(fmt, arg_list);
		va_end(arg_list);
	}

	void 
	CLog::
	vprintf(const char* fmt, va_list args) {
		const int buffer_size = 160;
		char stack_buffer[buffer_size+1];
		int nb_chars;

		nb_chars = vsnprintf(stack_buffer, buffer_size, fmt, args);

		if (nb_chars >= buffer_size) {
			char* heap_buffer = new char[nb_chars+1];

			vsnprintf(heap_buffer, nb_chars, fmt, args);
			add_string(heap_buffer);
			delete [] heap_buffer;
		} else {
			add_string(stack_buffer);
		}
	}

	const unsigned int
	CLog::
	get_nb_lines() {
		return m_nb_lines;
	}

	const char*
	CLog::
	get_line(const unsigned int n) {
		if (n < m_nb_lines) {
			return m_lines[n].c_str();
		} else {
			return "";
		}
	}

}
