/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: viewport.cpp,v 1.2 2003/08/03 15:40:56 rodzilla Exp $

    The Viewport class handles rendering in a rectangular part
    of the window.
    The main window refers to a viewport as well.
*/

#include <assert.h>
#include <math.h>
#include <SaM3d/display.h>
#include <SaM3d/opengl.h>
#include <SaM3d/viewport.h>

namespace SaM3d {

	using namespace std;

	// Current viewport is initialized to NULL.
	const Viewport* Viewport::m_current = NULL;

	Viewport::
	Viewport(int x, int y, int w, int h) {
		m_x = x;
		m_y = y;
		m_w = w;
		m_h = h;
	}

	// Get the current viewport.
	const Viewport&
	Viewport::
	current() {
		// The current viewport should not be NULL. The display's viewport is
		// activated as soon as the Display is created.
		assert (m_current != NULL);
		return *m_current;
	}

	// Activate a viewport.
	void
	Viewport::
	activate() const {
		// call glViewPort.
		glViewport(m_x, m_y, m_w, m_h);
		// this becomes the current viewport.
		m_current = (Viewport*)this;
	}

	void
	Viewport::
	move_resize(const int x, const int y, const int w, const int h) {
		m_x = x; m_y = y; m_w = w; m_h = h;
	}

	namespace {
		const Viewport* viewport_view_ortho = NULL;
	};

	void
	Viewport::
	begin_view_ortho() const {
		assert(viewport_view_ortho == NULL);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(0, m_w, 0, m_h, -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		viewport_view_ortho = this;
	}

	void
	Viewport::
	end_view_ortho() const {
		assert(viewport_view_ortho == this);
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		viewport_view_ortho = false;
	}


}

