/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: main.cpp,v 1.2 2003/08/03 16:05:57 rodzilla Exp $

    This exemple shows how to use viewports and cameras.
*/

#include <SaM3d/camera.h>
#include <SaM3d/console.h>
#include <SaM3d/display.h>
#include <SaM3d/log.h>
#include <SaM3d/loop.h>
#include <SaM3d/opengl.h>
#include <SaM3d/performance_counter.h>

using namespace SaM3d;

// Draw a lot of cubes.
void
drawcubes() {
	int x, y, z;

	for (x = -15; x <= 15; x += 6) for (y = -15; y <= 15; y += 6) for (z = -15; z <= 15; z += 6) {
		glPushMatrix();
		glTranslatef(x, y, z);
// Draw a cube (stolen in one of the first NeHe tutorials).
		glBegin(GL_QUADS);				// Start Drawing The Cube
		glColor3f(0.0f,1.0f,0.0f);			// Set The Color To Green
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Top)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Top)
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Bottom Left Of The Quad (Top)
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Bottom Right Of The Quad (Top)
		glColor3f(1.0f,0.5f,0.0f);			// Set The Color To Orange
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Top Right Of The Quad (Bottom)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Top Left Of The Quad (Bottom)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Bottom)
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Bottom)
		glColor3f(1.0f,0.0f,0.0f);			// Set The Color To Red
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Front)
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Front)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Front)
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Front)
		glColor3f(1.0f,1.0f,0.0f);			// Set The Color To Yellow
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Back)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Back)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Back)
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Back)
		glColor3f(0.0f,0.0f,1.0f);			// Set The Color To Blue
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Left)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Left)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Left)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Left)
		glColor3f(1.0f,0.0f,1.0f);			// Set The Color To Violet
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Right)
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Right)
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Right)
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Right)
		glEnd();				
		glPopMatrix();
	}
}

class MyCamera : public Camera {
public:
	void draw();
};

// This method draws 2 triangles and a line to show where a camera is.
// That's ugly, but this will show the camera's movements.
void
MyCamera::
draw() {
	glBegin(GL_LINES);
	m_position.glVertex();
	(m_position + m_forward).glVertex();
	glEnd();
	glPushMatrix();
	m_position.glTranslate();
	m_orientation.glRotate();
	glBegin(GL_TRIANGLES);
	glColor3f(1, 0, 0);
	glVertex3f(0, 0, 0);
	glColor3f(0, 1, 0);
	glVertex3f(-1, 0, 2);
	glVertex3f(1, 0, 2);
	glColor3f(1, 0, 0);
	glVertex3f(0, 0, 2);
	glColor3f(0, 0, 1);
	glVertex3f(0, 0, 2.5);
	glVertex3f(0, 0.5, 2.5);
	glEnd();
	glPopMatrix();
}

class Tutorial : public Loop {
	Viewport m_viewport_1;
	Viewport m_viewport_2;
	MyCamera m_camera[2];
	unsigned char m_current_camera;

	virtual bool handle_key_pressed(const SDL_keysym *keysym);
	virtual bool handle_mouse_motion(const int x, const int y);
	virtual bool handle_window_resize(const int w, const int h);

public:
	Tutorial();

	virtual void init();
	virtual void draw();
	virtual void move(const float dt);
};	

Tutorial::
Tutorial() : m_viewport_1(0, 0, Display.viewport().w()/2,  Display.viewport().h()),
	     m_viewport_2(Display.viewport().w()/2, 0, Display.viewport().w()/2,  Display.viewport().h()) {
	m_current_camera = 0;
	m_camera[0].set_fov(0.1, 200);
	m_camera[1].set_fov(0.1, 200);
}

// Init OpenGL parameters for this scene.
void
Tutorial::
init() {
	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDepthFunc(GL_LESS);
	glDisable(GL_CULL_FACE);
}

// This method draw the scene.
void 
Tutorial::
draw() {
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear the display.
	Texture::none().bind();
	m_viewport_1.activate();				// Activate the first (left) viewport.
	m_camera[0].activate();					// Activate the first camera.
	m_camera[1].draw();					// Draw the second camera.
	drawcubes();						// Draw cubes.
	m_viewport_2.activate();				// Activate the second (right) viewport.
	m_camera[1].activate();					// Activate the second camera.
	m_camera[0].draw();					// Draw the first camera.
	drawcubes();						// Draw cubes.

	Display.viewport().activate();				// Activate the main viewport.
	Console.draw();						// Draw the console.

	{							// Set the window title.
		char buf[64];

		sprintf(buf, "6 DOF camera tutorial. Frame rate: %i.", PerformanceCounter.fps());
		Display.set_window_title(buf);
	}
}

// Apply time-based movement to the current camera depending on the values of
// the forward_move, up_move and right_move variables.
void 
Tutorial::
move(float dt) {
	float forward_move = (key_state(SDLK_e)-key_state(SDLK_x));
	float right_move = (key_state(SDLK_d)-key_state(SDLK_s));
	float up_move = (key_state(SDLK_r)-key_state(SDLK_c));
	Vector3 tmp =   forward_move*m_camera[m_current_camera].forward()
		      + right_move*m_camera[m_current_camera].right()
		      + up_move*m_camera[m_current_camera].up();

	m_camera[m_current_camera].move(dt*50.0*tmp);
}

// Update the forward_move, up_move and right_move variables when a key is pressed.
bool
Tutorial::
handle_key_pressed(const SDL_keysym *keysym) {
	if (Console.handle_key_pressed(keysym)) return true;

	switch (keysym->sym) {
		case SDLK_SPACE:
			m_current_camera = 1-m_current_camera;
			break;
		default:
			return Loop::handle_key_pressed(keysym);
	}
	return true;
}

bool
Tutorial::
handle_mouse_motion(const int x, const int y) {
	if (mouse_button_state(SDL_BUTTON_RIGHT)) {		// If right mouse button is pressed the mouse
		if (x != 0) {					// rotates the camera around its forward vector.

			Vector3 v(0, 0, -1);			// This is the forward vector (in camera coordinates).
			Quaternion q(v, x*0.001);		// Create a quaternion that holds a rotation
								// around this vector.

			m_camera[m_current_camera].rotate(q);	// Rotate the camera.
		}
	} else {						// If right mouse button is not pressed the mouse
		if (x != 0) {					// rotates the camera around its up and right vectors.
			Vector3 v(0, 1, 0);
			Quaternion q(v, -x*0.001);

			m_camera[m_current_camera].rotate(q);
		}
		if (y != 0) {
			Vector3 v(1, 0, 0);
			Quaternion q(v, -y*0.001);

			m_camera[m_current_camera].rotate(q);
		}
	}
	return true;
}

// This method is invoked when the window is resized.
bool
Tutorial::
handle_window_resize(const int w, const int h) {
	// Resize both viewports.
	m_viewport_1.move_resize(0, 0, w/2, h);
	m_viewport_2.move_resize(w/2, 0, w/2, h);
	return true;
}

int main(int argc, char ** argv) {
	//	Initialize the display.
	Display;

	// Add a lot of text to the log to test bitmap font performance.
	for (int i = 0; i < 50; ++i) {
		Log.printf("SaM3d! Bitmap font performance test ! SaM3d! Bitmap font performance test !\n");
	}

	//	Create a loop for this tutorial.
	Tutorial t;
	//	Start the loop.
	t.start();

	//	Exit the program.
	return 0;
}

