/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: animated_texture.cpp,v 1.2 2003/08/24 15:45:00 rodzilla Exp $
*/

#include <SaM3d/loop.h>
#include "animated_texture.h"

namespace SaM3d {

	using namespace std;

	AnimatedTexture::
	AnimatedTexture(const float frequency,
			const Texture* textures,
			unsigned int nb_textures) : m_period(1000.0/frequency),
						    m_nb_textures(nb_textures),
						    m_textures(new Texture[m_nb_textures]),
						    m_current_texture(255) {
		unsigned int i;

		for (i = 0; i < m_nb_textures; ++i) {
			m_textures[i] = textures[i];
		}
	}

	AnimatedTexture::
	~AnimatedTexture() {
		delete [] m_textures;
	}

	bool
	AnimatedTexture::
	equal(const SaM3d::BaseTexture* t) const {
		return false;
	}

	unsigned char
	AnimatedTexture::
	hash() const {
		return 0;
	}

	void
	AnimatedTexture::
	create() {
		int texture_number = ((int)(Loop::current_time()/m_period))%m_nb_textures;

		if (texture_number != m_current_texture) {
			Texture& texture = m_textures[texture_number];

			texture.bind();
			m_id = texture.id();
			m_current_texture = texture_number;
		}

		m_status = ok;
	}

	void
	AnimatedTexture::
	release() {
		m_status = uninitialized;
	}

}

