/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: bsp.h,v 1.5 2003/08/31 10:04:34 rodzilla Exp $
*/

#ifndef __SAM3D_BSP_H__
#define __SAM3D_BSP_H__

#include <vector>
#include <SaM3d/math.h>
#include "bounding_volume.h"
#include "mesh.h"

namespace SaM3d {

	class BSP {
	public:
		class Node {
			struct node_info_t {
				Plane*	m_plane;
				Node*	m_front_node;
				Node*	m_back_node;

				node_info_t();
			};

			struct leaf_info_t {
				std::vector<Node*>	m_pvs;
				std::vector<O3d*>	m_meshes;
			};

			node_info_t*	m_node_info;
			leaf_info_t*	m_leaf_info;
			BoundingVolume*	m_bounding_volume;

			node_info_t& get_node_info();
			leaf_info_t& get_leaf_info();

		public:
			Node();

			void set_plane(Plane* p);
			void set_front_node(Node* n);
			void set_back_node(Node* n);

			void add_visible_leaf(Node* n);
			void add_mesh(O3d* m);

			void set_bounding_volume(BoundingVolume* v);

			Node* find_leaf_node(const Vector3& p);
			bool is_visible(const Frustum& f) const;
			std::vector<Node*>& get_visible_leaves();

			void draw(std::vector<O3d*>& transparent_meshes);
			void draw(const Frustum& f,
				  std::vector<O3d*>& transparent_meshes);
		};
	};

}

#endif
