/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: bsp_demo.cpp,v 1.15 2003/08/31 10:04:34 rodzilla Exp $
*/

#include <assert.h>
#include <stdlib.h>
#include <string>
#include <SaM3d/math.h>
#include <SaM3d/camera.h>
#include <SaM3d/display.h>
#include <SaM3d/console.h>
#include <SaM3d/lightmap.h>
#include <SaM3d/log.h>
#include <SaM3d/opengl.h>
#include <SaM3d/performance_counter.h>
#include "loader_bsp.h"
#include "bsp_demo.h"

using namespace std;
using namespace SaM3d;

static Camera camera;

Map* q3map;

BoolCfgVar DemoBsp::M_use_frustum("use_frustum", true);
BoolCfgVar DemoBsp::M_use_pvs("use_pvs", true);

DemoBsp::
DemoBsp(Dir& dir, const char* bspname) {
	q3map = Loader_BSP::read(dir, bspname);
	assert (q3map != NULL);

	/*
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	*/

	camera.set_fov(1, 3500);
}

DemoBsp::
~DemoBsp() {
}

void 
DemoBsp::
draw() {
	Frustum frustum;

	CHECK_OPENGL_ERROR;
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear the display.
	glLoadIdentity();
	camera.activate();
	frustum.update();

	CHECK_OPENGL_ERROR;

	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	CHECK_OPENGL_ERROR;
	OpenGL::cull_face(GL_FRONT);
	if (M_use_frustum) {
		if (M_use_pvs) {
			q3map->draw(camera.position(), frustum);
		} else {
			q3map->draw(frustum);
		}
	} else {
		if (M_use_pvs) {
			q3map->draw(camera.position());
		} else {
			q3map->draw();
		}
	}
	Texture::set_texture_unit(1);
	Texture::none().bind();
	Texture::set_texture_unit(0);
	Texture::none().bind();
	OpenGL::cull_face(GL_BACK);
	Console.draw();						// Draw the console.

        {                                                       // Set the window title.
                char buf[64];
 
                sprintf(buf, "BSP loader. Frame rate: %i. Triangles: %i",
			     PerformanceCounter.fps(),
			     PerformanceCounter.triangles());
                Display.set_window_title(buf);
        }
}

void 
DemoBsp::
move(float dt) {
	if (!Console.is_active()) {
		float forward_move = key_state(SDLK_e)-key_state(SDLK_x);
		float right_move = key_state(SDLK_d)-key_state(SDLK_s);
		float up_move = key_state(SDLK_r)-key_state(SDLK_c);
		Vector3 tmp =   forward_move*camera.forward()
			      + right_move*camera.right()
			      + up_move*camera.up();

		camera.move(dt*500.0*tmp);

		float gamma_change = key_state(SDLK_k)-key_state(SDLK_j);

		if (gamma_change != 0) {
			LightMap::gamma = LightMap::gamma+dt*gamma_change;
		}
	}
}

bool
DemoBsp::
handle_key_pressed(const SDL_keysym *keysym) {
	static bool wireframe = false;
	
	if (Console.handle_key_pressed(keysym)) return true;

	switch (keysym->sym) {
		case SDLK_o:
			M_use_frustum = !M_use_frustum;
			break;
		case SDLK_p:
			M_use_pvs = !M_use_pvs;
			break;
		case SDLK_w:
			wireframe = !wireframe;
			if (wireframe) {
				glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			} else {
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			}
			break;
		case SDLK_ESCAPE:
			exit();
			break;
		default:
			return Loop::handle_key_pressed(keysym);
	}
	return true;
}

bool
DemoBsp::
handle_mouse_motion(const int x, const int y) {
	if (mouse_button_state(SDL_BUTTON_RIGHT)) {
		if (x != 0) {
			Vector3 v(0, 0, 1);
			Quaternion q(v, -x*0.001);

			camera.rotate(q);
		}
	} else {
		if (x != 0) {
			Vector3 v(0, 1, 0);
			Quaternion q(v, -x*0.001);

			camera.rotate(q);
		}
		if (y != 0) {
			Vector3 v(1, 0, 0);
			Quaternion q(v, -y*0.001);

			camera.rotate(q);
		}
	}
	return true;
}

