/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: map.cpp,v 1.10 2003/08/31 10:04:34 rodzilla Exp $
*/

#include <string.h>
#include <unistd.h>
#include <iostream>
#include <SaM3d/file_texture.h>
#include <SaM3d/log.h>
#include <SaM3d/opengl.h>
#include "map.h"

namespace SaM3d {

	using namespace std;

	Map::
	Map() {
	}

	Map::
	~Map() {
	}

	void
	Map::
	draw() {
		vector<O3d*>::iterator i;

		for (i = m_mesh_l.begin(); i != m_mesh_l.end(); ++i) {
			(*i)->draw();
		}
	}

	void
	Map::
	draw(const Frustum& f) {
		vector<O3d*>::iterator i;

		for (i = m_mesh_l.begin(); i != m_mesh_l.end(); ++i) {
			(*i)->draw(f);
		}
	}

	namespace {
		vector<O3d*> transparent_meshes;
	}

	void
	Map::
	draw(const Vector3& p) {
		BSP::Node* node;

		node = m_bsp_tree->find_leaf_node(p);
		if (node != NULL) {
			vector<BSP::Node*>& visible_nodes = node->get_visible_leaves();
			
			if (!visible_nodes.empty()) {
				vector<BSP::Node*>::iterator i;
				vector<O3d*>::reverse_iterator j;

				transparent_meshes.clear();

				for (i = visible_nodes.begin(); i != visible_nodes.end(); ++i) {
					(*i)->draw(transparent_meshes);
				}
				
				for (j = transparent_meshes.rbegin(); j != transparent_meshes.rend(); ++j) {
					(*j)->draw();
				}

				return;
			}
		}

		draw();
	}

	void
	Map::
	draw(const Vector3& p, const Frustum& f) {
		BSP::Node* node;

		node = m_bsp_tree->find_leaf_node(p);
		if (node != NULL) {
			vector<BSP::Node*>& visible_nodes = node->get_visible_leaves();

			if (!visible_nodes.empty()) {
				vector<BSP::Node*>::iterator i;
				vector<O3d*>::reverse_iterator j;

				transparent_meshes.clear();

				for (i = visible_nodes.begin(); i != visible_nodes.end(); ++i) {
					(*i)->draw(f, transparent_meshes);
				}
				
				for (j = transparent_meshes.rbegin(); j != transparent_meshes.rend(); ++j) {
					(*j)->draw();
				}

				return;
			}
		}

		draw(f);
	}

}
