/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: mesh.h,v 1.5 2003/08/31 10:04:34 rodzilla Exp $
*/

#ifndef __SAM3D_MESH_H__
#define __SAM3D_MESH_H__

#include <stdio.h>
#include <vector>
#include <SaM3d/texture.h>
#include "o3d.h"
#include "shader.h"
#include "vertex_array.h"

namespace SaM3d {

	class Topology {
	public:
		enum primitive_t {
			none, triangles, triangle_strip, triangle_fan,
			raw_triangles, raw_triangle_strip, raw_triangle_fan,
			grid
		};

	protected:
		primitive_t		m_primitive;
		unsigned int*		m_indices;
		unsigned int		m_nb_indices;
		bool			m_own_indices;

		Topology();

	public:
		Topology(const primitive_t primitive,
			 unsigned int nb_indices,
			 unsigned int* indices = NULL);
		virtual ~Topology();

		unsigned int& operator[](const unsigned int index);
		bool operator==(const Topology& that) const;

		void draw();

		BoundingVolume* calculate_bounding_volume(VertexArray* va);
	};

	class Mesh : public O3d {
	protected:
		VertexArray*	m_vertex_array;
		Topology*	m_topology;

		Shader		m_shader;

	public:
		Mesh();
		virtual ~Mesh();

		void set_vertex_array(VertexArray* v);
		void set_topology(Topology* t);

		void set_shader(const Shader& shader);
		
		virtual void calculate_bounding_volume();

		virtual bool is_transparent() { return m_shader.is_transparent(); }

		virtual void draw();
	};

}

#endif
