/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: q3lexer.cpp,v 1.3 2003/08/31 10:04:34 rodzilla Exp $
*/

#include "q3lexer.h"

using namespace std;
using namespace SaM3d;

CQ3ShaderManager::Lexer::
Lexer(const Dir& d,
      const string& filename) : m_file(d, filename.c_str()) {
	m_next_chars = 0;
}

void
CQ3ShaderManager::Lexer::
fill_char_buffer() {
	for (;;) {
		switch (m_next_chars) {
			case 0: {
				m_next_char1 = m_file.get();
				m_next_char2 = m_file.get();
			}	break;
			case 1: {
				m_next_char1 = m_next_char2;
				m_next_char2 = m_file.get();
			}	break;
			case 2:
				break;
			default:
				abort();
				break;
		}

		m_next_chars = 2;

		if (m_next_char1 == '/' && m_next_char2 == '/') {
			char tmp;

			for (tmp = m_file.get(); tmp != '\n' && !eof(); tmp = m_file.get());
			m_next_char2 = tmp;
			m_next_chars = 1;
		} else {
			break;
		}
	}
}

char
CQ3ShaderManager::Lexer::
get() {
	fill_char_buffer();
	assert(m_next_chars == 2);
	m_next_chars = 1;
	return m_next_char1;
}

char
CQ3ShaderManager::Lexer::
look() {
	fill_char_buffer();
	assert(m_next_chars == 2);
	return m_next_char1;
}

void
CQ3ShaderManager::Lexer::
skip_spaces() {
	static string chars(" \t");
	int count = 0;

	for (; (int)(chars.find(look())) != -1 && !eof();) {
		get();
		count++;
	}
}

void
CQ3ShaderManager::Lexer::
skip_spaces_and_endl() {
	static string chars(" \t\n\15");
	int count = 0;

	for (; (int)(chars.find(look())) != -1 && !eof();) {
		get();
		count++;
	}
}

bool
CQ3ShaderManager::Lexer::
get_path(string& s) {
	static string chars("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-.$/*\\");
	bool rv = false;

	s = "";
	for (; (int)(chars.find(look())) != -1 && !eof();) {
		char c = get();

		if (c == '\\') {
			s += '/';
		} else {
			s += c;
		}
		rv = true;
	}
	skip_spaces();

	return rv;
}

bool
CQ3ShaderManager::Lexer::
get_identifier(string& s) {
	static string chars("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_");
	bool rv = false;

	s = "";
	for (; (int)(chars.find(look())) != -1 && !eof();) {
		s += get();
		rv = true;
	}
	skip_spaces();

	return rv;
}

bool
CQ3ShaderManager::Lexer::
get_float(float& f) {
	static string chars("1234567890+-.");
	string s;
	bool rv = false;

	for (; (int)(chars.find(look())) != -1 && !eof();) {
		s += get();
		rv = true;
	}
	f = atof(s.c_str());
	skip_spaces();

	return rv;
}

bool
CQ3ShaderManager::Lexer::
get_special(char c) {
	if (look() == c) {
		get();
		skip_spaces();
		return true;
	} else {
		return false;
	}
}

bool
CQ3ShaderManager::Lexer::
eof() {
	return m_file.eof();
}

