/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: q3shaders.cpp,v 1.11 2003/08/31 10:04:34 rodzilla Exp $
*/

#include <unistd.h>
#include <iostream>
#include <SaM3d/file.h>
#include <SaM3d/font.h>
#include <SaM3d/viewport.h>
#include "q3parser.h"

using namespace std;
using namespace SaM3d;

CQ3ShaderManager::
CQ3ShaderManager() {
	/*
	ComplexShader* q3simple = new ComplexShader;
	ComplexShader::Layer* layer;

	layer = new ComplexShader::Layer;
	layer->m_texture = ComplexShader::Layer::texture;
	layer->m_tcgen = ComplexShader::Layer::tc_texture;
	q3simple->add_layer(layer);

	layer = new ComplexShader::Layer;
	layer->m_texture = ComplexShader::Layer::lightmap;
	layer->m_tcgen = ComplexShader::Layer::tc_lightmap;
	layer->m_sfactor = GL_DST_COLOR;
	layer->m_dfactor = GL_ZERO;
	q3simple->add_layer(layer);
	*/

	m_shaders["q3simple"] = new Q3Shader();
	m_shaders["q3simple"]->shader = new SimpleShader();
	m_shaders["q3simple"]->sky = NULL;
}

CQ3ShaderManager&
CQ3ShaderManager::
instance() {
	static CQ3ShaderManager i;

	return i;
}


void
CQ3ShaderManager::
draw(const std::string& file_name, unsigned int percent) {
	Font font;
	const Viewport& vp = Viewport::current();
	const int width = vp.w();
	const int height = vp.h();
	const int h_height = height >> 1;
	const int h = height>>4;
	const int w = width-20;
	const float fontsize = width*0.03;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	vp.begin_view_ortho();
	Texture::none().bind();
	glColor4f(1, 1, 1, 1);
	glBegin(GL_QUADS);
	glVertex2f(10, h_height+5);
	glVertex2f(10+w*percent/100, h_height+5);
	glVertex2f(10+w*percent/100, h_height-5);
	glVertex2f(10, h_height-5);
	glEnd();
	glBegin(GL_LINE_STRIP);
	glVertex2f(8, h_height+7);
	glVertex2f(12+w, h_height+7);
	glVertex2f(12+w, h_height-7);
	glVertex2f(8, h_height-7);
	glVertex2f(8, h_height+7);
	glEnd();
	font.print(fontsize, h_height+h+7+fontsize, fontsize, "Loading shaders...");
	font.print(fontsize, h_height-h-7, fontsize, file_name.c_str());
	vp.end_view_ortho();

	SDL_GL_SwapBuffers();
	CHECK_OPENGL_ERROR;
}

void
CQ3ShaderManager::
load_shaders(Dir& q3dir) {
	set<string> file_list, shader_files;
	set<string>::iterator i;
	int nb_shader_files, current_shader_file;

	q3dir.get_file_list(file_list, "scripts");
	for (i = file_list.begin(); i != file_list.end(); ++i) {
		if ((*i).find(".shader") == (*i).size()-7) {
			shader_files.insert(*i);
		}
	}
	current_shader_file = 0;
	nb_shader_files = shader_files.size();
	for (i = shader_files.begin(); i != shader_files.end(); ++i) {
		draw(*i, current_shader_file*100/nb_shader_files);

		Parser parser(q3dir, *i);
		parser.parse();

		current_shader_file++;
	}
	draw("", 100);
}

Q3Shader*
CQ3ShaderManager::
get_shader(const string& name) {
	if (m_shaders.find(name) != m_shaders.end()) {
		return m_shaders[name];
	} else {
		return NULL;
	}
}

