/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: sky.cpp,v 1.1 2003/08/31 10:04:34 rodzilla Exp $

    The Mesh class handle meshes.
*/

#include <SaM3d/camera.h>
#include <SaM3d/log.h>
#include <SaM3d/opengl.h>
#include "mesh.h"
#include "sky.h"

namespace SaM3d {

	using namespace std;

	SkyBox::
	SkyBox(Dir& dir, const string& prefix) {
		static char* extensions[] = { ".jpg", ".tga", NULL };

		m_tex_ft = new FileTexture(dir, (prefix+"_ft").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
		m_tex_bk = new FileTexture(dir, (prefix+"_bk").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
		m_tex_lf = new FileTexture(dir, (prefix+"_lf").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
		m_tex_rt = new FileTexture(dir, (prefix+"_rt").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
		m_tex_up = new FileTexture(dir, (prefix+"_up").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
		m_tex_dn = new FileTexture(dir, (prefix+"_dn").c_str(), extensions, false, GL_CLAMP_TO_EDGE);
	}

	void
	SkyBox::
	draw() {
		OpenGL::cull_face(GL_NONE);
		glDisable(GL_BLEND);
		Texture::set_texture_unit(1);
		Texture::none().bind();
		Texture::set_texture_unit(0);
		glColor4f(1, 1, 1, 1);
		m_tex_ft.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(10, -10, -10);
		glTexCoord2f(0, 1); glVertex3f(10, 10, -10);
		glTexCoord2f(1, 0); glVertex3f(-10, -10, -10);
		glTexCoord2f(1, 1); glVertex3f(-10, 10, -10);
		glEnd();
		m_tex_bk.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(-10, -10, 10);
		glTexCoord2f(0, 1); glVertex3f(-10, 10, 10);
		glTexCoord2f(1, 0); glVertex3f(10, -10, 10);
		glTexCoord2f(1, 1); glVertex3f(10, 10, 10);
		glEnd();
		m_tex_lf.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(-10, -10, -10);
		glTexCoord2f(0, 1); glVertex3f(-10, 10, -10);
		glTexCoord2f(1, 0); glVertex3f(-10, -10, 10);
		glTexCoord2f(1, 1); glVertex3f(-10, 10, 10);
		glEnd();
		m_tex_rt.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(10, -10, 10);
		glTexCoord2f(0, 1); glVertex3f(10, 10, 10);
		glTexCoord2f(1, 0); glVertex3f(10, -10, -10);
		glTexCoord2f(1, 1); glVertex3f(10, 10, -10);
		glEnd();
		m_tex_up.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(10, 10, 10);
		glTexCoord2f(0, 1); glVertex3f(-10, 10, 10);
		glTexCoord2f(1, 0); glVertex3f(10, 10, -10);
		glTexCoord2f(1, 1); glVertex3f(-10, 10, -10);
		glEnd();
		/// TODO : I'm pretty sure there is a bug here...
		m_tex_dn.bind();
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(0, 0); glVertex3f(-10, -10, 10);
		glTexCoord2f(0, 1); glVertex3f(10, -10, 10);
		glTexCoord2f(1, 0); glVertex3f(-10, -10, -10);
		glTexCoord2f(1, 1); glVertex3f(10, -10, -10);
		glEnd();
	}


	class SkyDomeVertexArray : public VertexArray {
	public:
		SkyDomeVertexArray(float* data) {
			m_stride = 3*sizeof(float);
			m_data = NULL;
			m_vertices = (char*)data;
			m_tex_coords = (char*)data;
			m_lm_coords = NULL;
			m_normals = NULL;
			m_colors = NULL;
		}
	};

	SkyDome::
	SkyDome(const Shader& shader,
		float cloud_height) : m_cloud_height(0.25+cloud_height/1024.0),
				      m_shader(shader) {
	}

	void
	SkyDome::
	draw() {
		static float dome_coords[] = {
			0, 0, 0,
			-0.525731, 0.000000, 0.850651, 0.000000, 0.000000, 1.000000,
			-0.309017, 0.500000, 0.809017, 0.000000, 0.850651, 0.525731,
			0.309017, 0.500000, 0.809017, 0.525731, 0.000000, 0.850651,
			-0.809017, 0.309017, 0.500000, -0.850651, 0.525731, 0.000000,
			-0.500000, 0.809017, 0.309017, -0.500000, 0.809017, -0.309017,
			0.000000, 0.850651, -0.525731, 0.000000, 1.000000, 0.000000,
			0.500000, 0.809017, 0.309017, 0.500000, 0.809017, -0.309017,
			0.850651, 0.525731, 0.000000, 0.809017, 0.309017, 0.500000,
			1.000000, 0.000000, 0.000000, 0.850651, -0.525731, 0.000000,
			0.809017, -0.309017, 0.500000, 0.809017, 0.309017, -0.500000,
			0.525731, 0.000000, -0.850651, 0.809017, -0.309017, -0.500000,
			0.309017, 0.500000, -0.809017, -0.309017, 0.500000, -0.809017,
			-0.525731, 0.000000, -0.850651, 0.000000, 0.000000, -1.000000,
			-0.309017, -0.500000, -0.809017, 0.000000, -0.850651, -0.525731,
			0.309017, -0.500000, -0.809017, 0.500000, -0.809017, -0.309017,
			0.000000, -1.000000, 0.000000, 0.000000, -0.850651, 0.525731,
			0.500000, -0.809017, 0.309017, -0.500000, -0.809017, -0.309017,
			-0.850651, -0.525731, 0.000000, -0.500000, -0.809017, 0.309017,
			-0.809017, -0.309017, 0.500000, -0.309017, -0.500000, 0.809017,
			0.309017, -0.500000, 0.809017, -1.000000, 0.000000, 0.000000,
			-0.809017, 0.309017, -0.500000, -0.809017, -0.309017, -0.500000
		};

		static unsigned int dome_elems[] = {
			1, 2, 3, 4, 3, 5, 6, 5, 2, 3, 2, 5, 1, 3, 7,
			8, 7, 9, 4, 9, 3, 7, 3, 9, 8, 9, 10, 11, 10, 12,
			4, 12, 9, 10, 9, 12, 4, 13, 12, 11, 12, 14, 15, 14, 13,
			12, 13, 14, 4, 5, 13, 15, 13, 16, 6, 16, 5, 13, 5, 16,
			15, 16, 17, 18, 17, 19, 6, 19, 16, 17, 16, 19, 15, 17, 20,
			21, 20, 22, 18, 22, 17, 20, 17, 22, 11, 14, 23, 21, 23, 20,
			15, 20, 14, 23, 14, 20, 11, 23, 24, 25, 24, 26, 21, 26, 23,
			24, 23, 26, 25, 26, 27, 28, 27, 29, 21, 29, 26, 27, 26, 29,
			28, 29, 30, 18, 30, 22, 21, 22, 29, 30, 29, 22, 28, 30, 31,
			32, 31, 33, 18, 33, 30, 31, 30, 33, 28, 31, 34, 35, 34, 36,
			32, 36, 31, 34, 31, 36, 35, 36, 37, 1, 37, 38, 32, 38, 36,
			37, 36, 38, 1, 38, 2, 6, 2, 39, 32, 39, 38, 2, 38, 39,
			32, 33, 39, 6, 39, 19, 18, 19, 33, 39, 33, 19, 8, 40, 7,
			1, 7, 37, 35, 37, 40, 7, 40, 37, 8, 41, 40, 35, 40, 42,
			25, 42, 41, 40, 41, 42, 8, 10, 41, 25, 41, 24, 11, 24, 10,
			41, 10, 24, 28, 34, 27, 25, 27, 42, 35, 42, 34, 27, 34, 42
		};

		static SkyDomeVertexArray va(dome_coords);
		static Topology top(Topology::triangles, 240, dome_elems);

		OpenGL::cull_face(GL_NONE);
		glPushMatrix();
		glScalef(256, 256, 256);
		glRotatef(90, 1, 0, 0);
		glTranslatef(0, 0, m_cloud_height);
		CHECK_OPENGL_ERROR;
		m_shader.draw(&va, &top);
		glPopMatrix();
	}

	Sky::
	Sky(SkyBox* near, SkyDome* clouds, SkyBox* far) : m_far(far),
							  m_clouds(clouds),
							  m_near(near) {
	}

	void
	Sky::
	draw() {
		if (Camera::current() == NULL) return;

		glPushMatrix();
		Camera::current()->get_position().glTranslate();

		glDepthRange(1, 1);
		CHECK_OPENGL_ERROR;
		if (m_far) {
			m_far->draw();
		}
		if (m_clouds) {
			m_clouds->draw();
		}
		if (m_near) {
			m_near->draw();
		}
		glDepthRange(0, 1);
		CHECK_OPENGL_ERROR;

		glPopMatrix();
	}
}

