/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: vertex.h,v 1.1.1.1 2003/07/25 19:28:23 rodzilla Exp $
*/

#ifndef __SAM3D_VERTEX_H__
#define __SAM3D_VERTEX_H__

#include <SaM3d/math.h>
#include "color.h"

class Vertex {
public:
	SaM3d::Vector3 vertex;

	void glSpecify() const {
		glVertex3fv((float*)&vertex);
	}
};

class VertexN {
public:
	SaM3d::Vector3 normal;
	SaM3d::Vector3 vertex;

	void glSpecify() const {
		glNormal3fv((float*)&normal);
		glVertex3fv((float*)&vertex);
	}
};

std::ostream& operator <<(std::ostream& s, const VertexN& v);

class VertexNT {
public:
	SaM3d::Vector2 tex_coord;
	SaM3d::Vector3 normal;
	SaM3d::Vector3 vertex;

	bool operator ==(const VertexNT& v) const {
		return (tex_coord == v.tex_coord) &&
		       (normal == v.normal) &&
		       (vertex == v.vertex);
	}

	void glSpecify() const {
		glTexCoord2fv((float*)&tex_coord);
		glNormal3fv((float*)&normal);
		glVertex3fv((float*)&vertex);
	}
};

std::ostream& operator <<(std::ostream& s, const VertexNT& v);

class VertexTLmNC {
public:
	SaM3d::Vector3 vertex;
	SaM3d::Vector2 tex_coord;
	SaM3d::Vector2 lightmap_coord;
	SaM3d::Vector3 normal;
	Color color;

	bool operator ==(const VertexTLmNC& v) const {
		return (tex_coord == v.tex_coord) &&
		       (lightmap_coord == v.lightmap_coord) &&
		       (color == v.color) &&
		       (normal == v.normal) &&
		       (vertex == v.vertex);
	}

	VertexTLmNC& operator =(const VertexTLmNC& v) {
		vertex = v.vertex;
		tex_coord = v.tex_coord;
		lightmap_coord = v.lightmap_coord;
		normal = v.normal;
		color = v.color;

		return *this;
	}

	VertexTLmNC& operator +=(const VertexTLmNC& v) {
		vertex += v.vertex;
		tex_coord += v.tex_coord;
		lightmap_coord += v.lightmap_coord;
		normal += v.normal;
		color += v.color;

		return *this;
	}

	void glSpecify() const {
		glTexCoord2fv((float*)&tex_coord);
		glNormal3fv((float*)&normal);
		glVertex3fv((float*)&vertex);
	}
};

VertexTLmNC operator *(const float x, const VertexTLmNC& v);

#endif
