/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: vertex_array.cpp,v 1.6 2003/08/31 10:04:34 rodzilla Exp $
*/

#include <SaM3d/texture.h>
#include "vertex_array.h"

namespace SaM3d {

	using namespace std;

	VertexArray*
	VertexArray::
	sm_current_array = NULL;

	unsigned int
	VertexArray::
	sm_current_index = 0;

	char
	VertexArray::
	sm_current_mode = -1;

	VertexArray::
	VertexArray() {
	}

	VertexArray::
	VertexArray(const unsigned int size,
		    const unsigned int contents) : m_contents(contents) {
		char* ptr;

		m_stride = 0;
		if (contents & vertices) {
			m_stride += sizeof(Vector3);
		}
		if (contents & tex_coords) {
			m_stride += sizeof(Vector2);
		}
		if (contents & lm_coords) {
			m_stride += sizeof(Vector2);
		}
		if (contents & normals) {
			m_stride += sizeof(Vector3);
		}
		if (contents & colors) {
			m_stride += sizeof(Color);
		}

		ptr = m_data = new char[size*m_stride];

		if (contents & vertices) {
			m_vertices = ptr;
			ptr += sizeof(Vector3);
		} else {
			m_vertices = NULL;
		}
		if (contents & tex_coords) {
			m_tex_coords = ptr;
			ptr += sizeof(Vector2);
		} else {
			m_tex_coords = NULL;
		}
		if (contents & lm_coords) {
			m_lm_coords = ptr;
			ptr += sizeof(Vector2);
		} else {
			m_lm_coords = NULL;
		}
		if (contents & normals) {
			m_normals = ptr;
			ptr += sizeof(Vector3);
		} else {
			m_normals = NULL;
		}
		if (contents & colors) {
			m_colors = ptr;
		} else {
			m_colors = NULL;
		}

	}

	VertexArray::
	VertexArray(const VertexArray& src,
		    const unsigned int first_index) : m_contents(src.m_contents),
						      m_vertices(src.m_vertices),
						      m_tex_coords(src.m_tex_coords),
						      m_lm_coords(src.m_lm_coords),
						      m_normals(src.m_normals),
						      m_colors(src.m_colors),
						      m_stride(src.m_stride) {
		if (m_vertices) {
			m_vertices += first_index*m_stride;
		}
		if (m_tex_coords) {
			m_tex_coords += first_index*m_stride;
		}
		if (m_lm_coords) {
			m_lm_coords += first_index*m_stride;
		}
		if (m_normals) {
			m_normals += first_index*m_stride;
		}
		if (m_colors) {
			m_colors += first_index*m_stride;
		}
	}

	VertexArray::
	VertexArray(const VertexArray& vertex_array,
		    const unsigned int inherited_contents,
		    const unsigned int contents) {
	}

	VertexArray::
	~VertexArray() {
	}

	Vector3&
	VertexArray::
	vertex(const unsigned int index) {
		Vector3* rv = (Vector3*)(m_vertices+m_stride*index);
		return *rv;
	}

	Vector2&
	VertexArray::
	tex_coord(const unsigned int index) {
		Vector2* rv = (Vector2*)(m_tex_coords+m_stride*index);
		return *rv;
	}

	Vector2&
	VertexArray::
	lm_coord(const unsigned int index) {
		Vector2* rv = (Vector2*)(m_lm_coords+m_stride*index);
		return *rv;
	}

	Vector3&
	VertexArray::
	normal(const unsigned int index) {
		Vector3* rv = (Vector3*)(m_normals+m_stride*index);
		return *rv;
	}

	Color&
	VertexArray::
	color(const unsigned int index) {
		Color* rv = (Color*)(m_colors+m_stride*index);
		return *rv;
	}

	void
	VertexArray::
	glPointer(const unsigned int index) {
		if (   (sm_current_array != this)
		    || (sm_current_index != index)
		    || (sm_current_mode != 0)) {
			sm_current_array = this;
			sm_current_index = index;
			sm_current_mode = 0;
		} else {
			return;
		}

		if (m_vertices) {
			glVertexPointer(3, GL_FLOAT, m_stride,
					m_vertices+m_stride*index);
			glEnableClientState(GL_VERTEX_ARRAY);
		} else {
			glDisableClientState(GL_VERTEX_ARRAY);
		}

		if (m_tex_coords) {
			glTexCoordPointer(2, GL_FLOAT, m_stride,
					  m_tex_coords+m_stride*index);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		} else {
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}

		Texture::set_texture_unit(1);
		if (m_lm_coords) {
			glTexCoordPointer(2, GL_FLOAT, m_stride,
					  m_lm_coords+m_stride*index);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		} else {
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}
		Texture::set_texture_unit(0);

		if (m_normals) {
			glNormalPointer(GL_FLOAT, m_stride,
					m_normals+m_stride*index);
			glEnableClientState(GL_NORMAL_ARRAY);
		} else {
			glDisableClientState(GL_NORMAL_ARRAY);
		}

		if (m_colors) {
			glColorPointer(4, GL_FLOAT, m_stride,
				       m_colors+m_stride*index);
			glEnableClientState(GL_COLOR_ARRAY);
		} else {
			glDisableClientState(GL_COLOR_ARRAY);
		}
	}

	void
	VertexArray::
	glPointer_texture(const unsigned int index) {
		if (   (sm_current_array != this)
		    || (sm_current_index != index)
		    || (sm_current_mode != 1)) {
			sm_current_array = this;
			sm_current_index = index;
			sm_current_mode = 1;
		} else {
			return;
		}

		if (m_vertices) {
			glVertexPointer(3, GL_FLOAT, m_stride,
					m_vertices+m_stride*index);
			glEnableClientState(GL_VERTEX_ARRAY);
		} else {
			glDisableClientState(GL_VERTEX_ARRAY);
		}

		Texture::set_texture_unit(1);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		Texture::set_texture_unit(0);

		if (m_tex_coords) {
			glTexCoordPointer(2, GL_FLOAT, m_stride,
					  m_tex_coords+m_stride*index);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		} else {
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}

		if (m_normals) {
			glNormalPointer(GL_FLOAT, m_stride,
					m_normals+m_stride*index);
			glEnableClientState(GL_NORMAL_ARRAY);
		} else {
			glDisableClientState(GL_NORMAL_ARRAY);
		}

		if (m_colors) {
			glColorPointer(4, GL_FLOAT, m_stride,
				       m_colors+m_stride*index);
			glEnableClientState(GL_COLOR_ARRAY);
		} else {
			glDisableClientState(GL_COLOR_ARRAY);
		}
	}

	void
	VertexArray::
	glPointer_lightmap(const unsigned int index) {
		if (   (sm_current_array != this)
		    || (sm_current_index != index)
		    || (sm_current_mode != 2)) {
			sm_current_array = this;
			sm_current_index = index;
			sm_current_mode = 2;
		} else {
			return;
		}

		if (m_vertices) {
			glVertexPointer(3, GL_FLOAT, m_stride,
					m_vertices+m_stride*index);
			glEnableClientState(GL_VERTEX_ARRAY);
		} else {
			glDisableClientState(GL_VERTEX_ARRAY);
		}

		Texture::set_texture_unit(1);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		Texture::set_texture_unit(0);

		if (m_lm_coords) {
			glTexCoordPointer(2, GL_FLOAT, m_stride,
					  m_lm_coords+m_stride*index);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		} else {
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}

		if (m_normals) {
			glNormalPointer(GL_FLOAT, m_stride,
					m_normals+m_stride*index);
			glEnableClientState(GL_NORMAL_ARRAY);
		} else {
			glDisableClientState(GL_NORMAL_ARRAY);
		}

		if (m_colors) {
			glColorPointer(4, GL_FLOAT, m_stride,
				       m_colors+m_stride*index);
			glEnableClientState(GL_COLOR_ARRAY);
		} else {
			glDisableClientState(GL_COLOR_ARRAY);
		}
	}

	bool
	VertexArray::
	aligned(unsigned int a, unsigned int b, unsigned int c) {
		assert(a != b && a != c && b != c);

		if (m_contents & vertices) {
			Vector3 ab = vertex(b) - vertex(a);
			Vector3 ac = vertex(c) - vertex(a);

			if (abs(ab.x*ac.y-ab.y*ac.x) > 0.01) return false;
			if (abs(ab.x*ac.z-ab.z*ac.x) > 0.01) return false;
			if (abs(ab.y*ac.z-ab.z*ac.y) > 0.01) return false;
		}

		return true;
	}











	VertexArray4BezierPatch::
	VertexArray4BezierPatch(VertexArray& src, 
				const unsigned hsize,
				const unsigned vsize,
				const unsigned hsub,
				const unsigned vsub) : VertexArray((((hsize-1)>>1)*hsub+1)*(((vsize)>>1)*vsub+1),
					                           src.contents()) {
		unsigned gridhsize = ((hsize-1)>>1)*hsub+1;
		unsigned gridvsize = ((vsize-1)>>1)*vsub+1;
		unsigned i, j;
		float fhsub = 1.0/hsub;
		float fvsub = 1.0/vsub;

		for (i = 0; i < gridvsize; i++) {
			unsigned pi = (i / vsub) << 1;
			unsigned di = i % vsub;
			float fdi = di*fvsub;

			for (j = 0; j < gridhsize; j++) {
				unsigned pj = (j / hsub) << 1;
				unsigned dj = j % hsub;
				float fdj = dj*fhsub;

				if (di == 0) {
					if (dj == 0) {
						if (m_contents & vertices) {
							vertex(i*gridhsize+j) = src.vertex(pi*hsize+pj);
						}
						if (m_contents & tex_coords) {
							tex_coord(i*gridhsize+j) = src.tex_coord(pi*hsize+pj);
						}
						if (m_contents & lm_coords) {
							lm_coord(i*gridhsize+j) = src.lm_coord(pi*hsize+pj);
						}
						if (m_contents & normals) {
							normal(i*gridhsize+j) = src.normal(pi*hsize+pj);
						}
						if (m_contents & colors) {
							color(i*gridhsize+j) = src.color(pi*hsize+pj);
						}
					} else {
						if (m_contents & vertices) {
							vertex(i*gridhsize+j) =  (1-fdj)*(1-fdj)*src.vertex(pi*hsize+pj);
							vertex(i*gridhsize+j) += 2*(1-fdj)*fdj*src.vertex(pi*hsize+pj+1);
							vertex(i*gridhsize+j) += fdj*fdj*src.vertex(pi*hsize+pj+2);
						}
						if (m_contents & tex_coords) {
							tex_coord(i*gridhsize+j) =  (1-fdj)*(1-fdj)*src.tex_coord(pi*hsize+pj);
							tex_coord(i*gridhsize+j) += 2*(1-fdj)*fdj*src.tex_coord(pi*hsize+pj+1);
							tex_coord(i*gridhsize+j) += fdj*fdj*src.tex_coord(pi*hsize+pj+2);
						}
						if (m_contents & lm_coords) {
							lm_coord(i*gridhsize+j) =  (1-fdj)*(1-fdj)*src.lm_coord(pi*hsize+pj);
							lm_coord(i*gridhsize+j) += 2*(1-fdj)*fdj*src.lm_coord(pi*hsize+pj+1);
							lm_coord(i*gridhsize+j) += fdj*fdj*src.lm_coord(pi*hsize+pj+2);
						}
						if (m_contents & normals) {
							normal(i*gridhsize+j) =  (1-fdj)*(1-fdj)*src.normal(pi*hsize+pj);
							normal(i*gridhsize+j) += 2*(1-fdj)*fdj*src.normal(pi*hsize+pj+1);
							normal(i*gridhsize+j) += fdj*fdj*src.normal(pi*hsize+pj+2);
						}
						if (m_contents & colors) {
							color(i*gridhsize+j) =  (1-fdj)*(1-fdj)*src.color(pi*hsize+pj);
							color(i*gridhsize+j) += 2*(1-fdj)*fdj*src.color(pi*hsize+pj+1);
							color(i*gridhsize+j) += fdj*fdj*src.color(pi*hsize+pj+2);
						}
					}
				} else if (dj == 0) {
					if (m_contents & vertices) {
						vertex(i*gridhsize+j) =  (1-fdi)*(1-fdi)*src.vertex(pi*hsize+pj);
						vertex(i*gridhsize+j) += 2*(1-fdi)*fdi*src.vertex((pi+1)*hsize+pj);
						vertex(i*gridhsize+j) += fdi*fdi*src.vertex((pi+2)*hsize+pj);
					}
					if (m_contents & tex_coords) {
						tex_coord(i*gridhsize+j) =  (1-fdi)*(1-fdi)*src.tex_coord(pi*hsize+pj);
						tex_coord(i*gridhsize+j) += 2*(1-fdi)*fdi*src.tex_coord((pi+1)*hsize+pj);
						tex_coord(i*gridhsize+j) += fdi*fdi*src.tex_coord((pi+2)*hsize+pj);
					}
					if (m_contents & lm_coords) {
						lm_coord(i*gridhsize+j) =  (1-fdi)*(1-fdi)*src.lm_coord(pi*hsize+pj);
						lm_coord(i*gridhsize+j) += 2*(1-fdi)*fdi*src.lm_coord((pi+1)*hsize+pj);
						lm_coord(i*gridhsize+j) += fdi*fdi*src.lm_coord((pi+2)*hsize+pj);
					}
					if (m_contents & normals) {
						normal(i*gridhsize+j) =  (1-fdi)*(1-fdi)*src.normal(pi*hsize+pj);
						normal(i*gridhsize+j) += 2*(1-fdi)*fdi*src.normal((pi+1)*hsize+pj);
						normal(i*gridhsize+j) += fdi*fdi*src.normal((pi+2)*hsize+pj);
					}
					if (m_contents & colors) {
						color(i*gridhsize+j) =  (1-fdi)*(1-fdi)*src.color(pi*hsize+pj);
						color(i*gridhsize+j) += 2*(1-fdi)*fdi*src.color((pi+1)*hsize+pj);
						color(i*gridhsize+j) += fdi*fdi*src.color((pi+2)*hsize+pj);
					}
				} else {
					if (m_contents & vertices) {
						vertex(i*gridhsize+j) =  (1-fdi)*(1-fdi)*(1-fdj)*(1-fdj)*src.vertex(pi*hsize+pj);
						vertex(i*gridhsize+j) += (1-fdi)*(1-fdi)*2*fdj*(1-fdj)*src.vertex(pi*hsize+pj+1);
						vertex(i*gridhsize+j) += (1-fdi)*(1-fdi)*fdj*fdj*src.vertex(pi*hsize+pj+2);
						vertex(i*gridhsize+j) += 2*fdi*(1-fdi)*(1-fdj)*(1-fdj)*src.vertex((pi+1)*hsize+pj);
						vertex(i*gridhsize+j) += 4*fdi*(1-fdi)*fdj*(1-fdj)*src.vertex((pi+1)*hsize+pj+1);
						vertex(i*gridhsize+j) += 2*fdi*(1-fdi)*fdj*fdj*src.vertex((pi+1)*hsize+pj+2);
						vertex(i*gridhsize+j) += fdi*fdi*(1-fdj)*(1-fdj)*src.vertex((pi+2)*hsize+pj);
						vertex(i*gridhsize+j) += fdi*fdi*2*fdj*(1-fdj)*src.vertex((pi+2)*hsize+pj+1);
						vertex(i*gridhsize+j) += fdi*fdi*fdj*fdj*src.vertex((pi+2)*hsize+pj+2);
					}
					if (m_contents & tex_coords) {
						tex_coord(i*gridhsize+j) =  (1-fdi)*(1-fdi)*(1-fdj)*(1-fdj)*src.tex_coord(pi*hsize+pj);
						tex_coord(i*gridhsize+j) += (1-fdi)*(1-fdi)*2*fdj*(1-fdj)*src.tex_coord(pi*hsize+pj+1);
						tex_coord(i*gridhsize+j) += (1-fdi)*(1-fdi)*fdj*fdj*src.tex_coord(pi*hsize+pj+2);
						tex_coord(i*gridhsize+j) += 2*fdi*(1-fdi)*(1-fdj)*(1-fdj)*src.tex_coord((pi+1)*hsize+pj);
						tex_coord(i*gridhsize+j) += 4*fdi*(1-fdi)*fdj*(1-fdj)*src.tex_coord((pi+1)*hsize+pj+1);
						tex_coord(i*gridhsize+j) += 2*fdi*(1-fdi)*fdj*fdj*src.tex_coord((pi+1)*hsize+pj+2);
						tex_coord(i*gridhsize+j) += fdi*fdi*(1-fdj)*(1-fdj)*src.tex_coord((pi+2)*hsize+pj);
						tex_coord(i*gridhsize+j) += fdi*fdi*2*fdj*(1-fdj)*src.tex_coord((pi+2)*hsize+pj+1);
						tex_coord(i*gridhsize+j) += fdi*fdi*fdj*fdj*src.tex_coord((pi+2)*hsize+pj+2);
					}
					if (m_contents & lm_coords) {
						lm_coord(i*gridhsize+j) =  (1-fdi)*(1-fdi)*(1-fdj)*(1-fdj)*src.lm_coord(pi*hsize+pj);
						lm_coord(i*gridhsize+j) += (1-fdi)*(1-fdi)*2*fdj*(1-fdj)*src.lm_coord(pi*hsize+pj+1);
						lm_coord(i*gridhsize+j) += (1-fdi)*(1-fdi)*fdj*fdj*src.lm_coord(pi*hsize+pj+2);
						lm_coord(i*gridhsize+j) += 2*fdi*(1-fdi)*(1-fdj)*(1-fdj)*src.lm_coord((pi+1)*hsize+pj);
						lm_coord(i*gridhsize+j) += 4*fdi*(1-fdi)*fdj*(1-fdj)*src.lm_coord((pi+1)*hsize+pj+1);
						lm_coord(i*gridhsize+j) += 2*fdi*(1-fdi)*fdj*fdj*src.lm_coord((pi+1)*hsize+pj+2);
						lm_coord(i*gridhsize+j) += fdi*fdi*(1-fdj)*(1-fdj)*src.lm_coord((pi+2)*hsize+pj);
						lm_coord(i*gridhsize+j) += fdi*fdi*2*fdj*(1-fdj)*src.lm_coord((pi+2)*hsize+pj+1);
						lm_coord(i*gridhsize+j) += fdi*fdi*fdj*fdj*src.lm_coord((pi+2)*hsize+pj+2);
					}
					if (m_contents & normals) {
						normal(i*gridhsize+j) =  (1-fdi)*(1-fdi)*(1-fdj)*(1-fdj)*src.normal(pi*hsize+pj);
						normal(i*gridhsize+j) += (1-fdi)*(1-fdi)*2*fdj*(1-fdj)*src.normal(pi*hsize+pj+1);
						normal(i*gridhsize+j) += (1-fdi)*(1-fdi)*fdj*fdj*src.normal(pi*hsize+pj+2);
						normal(i*gridhsize+j) += 2*fdi*(1-fdi)*(1-fdj)*(1-fdj)*src.normal((pi+1)*hsize+pj);
						normal(i*gridhsize+j) += 4*fdi*(1-fdi)*fdj*(1-fdj)*src.normal((pi+1)*hsize+pj+1);
						normal(i*gridhsize+j) += 2*fdi*(1-fdi)*fdj*fdj*src.normal((pi+1)*hsize+pj+2);
						normal(i*gridhsize+j) += fdi*fdi*(1-fdj)*(1-fdj)*src.normal((pi+2)*hsize+pj);
						normal(i*gridhsize+j) += fdi*fdi*2*fdj*(1-fdj)*src.normal((pi+2)*hsize+pj+1);
						normal(i*gridhsize+j) += fdi*fdi*fdj*fdj*src.normal((pi+2)*hsize+pj+2);
					}
					if (m_contents & colors) {
						color(i*gridhsize+j) =  (1-fdi)*(1-fdi)*(1-fdj)*(1-fdj)*src.color(pi*hsize+pj);
						color(i*gridhsize+j) += (1-fdi)*(1-fdi)*2*fdj*(1-fdj)*src.color(pi*hsize+pj+1);
						color(i*gridhsize+j) += (1-fdi)*(1-fdi)*fdj*fdj*src.color(pi*hsize+pj+2);
						color(i*gridhsize+j) += 2*fdi*(1-fdi)*(1-fdj)*(1-fdj)*src.color((pi+1)*hsize+pj);
						color(i*gridhsize+j) += 4*fdi*(1-fdi)*fdj*(1-fdj)*src.color((pi+1)*hsize+pj+1);
						color(i*gridhsize+j) += 2*fdi*(1-fdi)*fdj*fdj*src.color((pi+1)*hsize+pj+2);
						color(i*gridhsize+j) += fdi*fdi*(1-fdj)*(1-fdj)*src.color((pi+2)*hsize+pj);
						color(i*gridhsize+j) += fdi*fdi*2*fdj*(1-fdj)*src.color((pi+2)*hsize+pj+1);
						vertex(i*gridhsize+j) += fdi*fdi*fdj*fdj*src.vertex((pi+2)*hsize+pj+2);
					}
				}
			}
		}
	}

}
