/**************************************************************************

    This file is part of "The Science and Magick 3d Engine", yet
    another OpenGL-based engine.
    Copyright (C) 2001 - 2002 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: vertex_array.h,v 1.3 2003/08/19 17:49:26 rodzilla Exp $
*/

#ifndef __SAM3D_VERTEX_ARRAY_H__
#define __SAM3D_VERTEX_ARRAY_H__

#include "color.h"
#include <SaM3d/math.h>

namespace SaM3d {

	class VertexArray {
		static VertexArray*	sm_current_array;
		static unsigned int	sm_current_index;
		static char		sm_current_mode;

	protected:
		unsigned int	m_size;
		unsigned int	m_contents;
		char*		m_data;

		char*		m_vertices;
		char*		m_tex_coords;
		char*		m_lm_coords;
		char*		m_normals;
		char*		m_colors;
		int		m_stride;

		VertexArray();

	public:
		enum contents_t {
			vertices	= 1,
			tex_coords	= 2,
			lm_coords	= 4,
			normals		= 8,
			colors		= 16
		};

		VertexArray(const unsigned int size,
			    const unsigned int contents);

		VertexArray(const VertexArray& vertex_array,
			    const unsigned int first_index = 0);

		VertexArray(const VertexArray& vertex_array,
			    const unsigned int inherited_contents,
			    const unsigned int contents);

		~VertexArray();

		unsigned int contents() const { return m_contents; }

		Vector3& vertex(const unsigned int index);
		Vector2& tex_coord(const unsigned int index);
		Vector2& lm_coord(const unsigned int index);
		Vector3& normal(const unsigned int index);
		Color&   color(const unsigned int index);

		void glPointer(const unsigned int index = 0);
		void glPointer_texture(const unsigned int index = 0);
		void glPointer_lightmap(const unsigned int index = 0);

		bool aligned(unsigned int a, unsigned int b, unsigned int c);
	};

	class VertexArray4BezierPatch : public VertexArray {
	public:
		VertexArray4BezierPatch(VertexArray& vertex_array, 
					const unsigned hsize,
					const unsigned vsize,
					const unsigned hsub,
					const unsigned vsub);
	};

}

#endif
