/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: lesson_04.cpp,v 1.1.1.1 2003/07/25 19:28:30 rodzilla Exp $
*/

#include <SaM3d/display.h>
#include <SaM3d/opengl.h>
#include <SaM3d/performance_counter.h>
#include "lesson_04.h"

using namespace SaM3d;

Lesson04::
Lesson04() {
	m_camera.set_fov(0.1, 200);

	m_angle_triangle = 0;
	m_angle_quad = 0;
}

// Init OpenGL parameters for this scene.
void
Lesson04::
init() {
	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDepthFunc(GL_LESS);
	glDisable(GL_CULL_FACE);
}

// Init OpenGL parameters for this scene.
void
Lesson04::
draw() {
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear the display.

	m_camera.activate();

	glPushMatrix();					//Store The Current Modelview Matrix
	glTranslatef(-1.5f,0.0f,-6.0f);			// Move Left 1.5 Units And Into The Screen 6.0
	glRotatef(m_angle_triangle,0.0f,1.0f,0.0f);	// Rotate The Triangle On The Y axis ( NEW )

	glBegin(GL_TRIANGLES);				// Drawing Using Triangles
	glColor3f(1.0f,0.0f,0.0f);			// Set The Color To Red
	glVertex3f( 0.0f, 1.0f, 0.0f);			// Move Up One Unit From Center (Top Point)
	glColor3f(0.0f,1.0f,0.0f);			// Set The Color To Green
	glVertex3f(-1.0f,-1.0f, 0.0f);			// Left And Down One Unit (Bottom Left)
	glColor3f(0.0f,0.0f,1.0f);			// Set The Color To Blue
	glVertex3f( 1.0f,-1.0f, 0.0f);			// Right And Down One Unit (Bottom Right)
	glEnd();					// Finished Drawing The Triangle			

	glPopMatrix();					// Restore The Current Modelview Matrix
	glTranslatef(1.5f,0.0f,-6.0f);			// Move Right 1.5 Units And Into The Screen 6.0
	glRotatef(m_angle_quad,1.0f,0.0f,0.0f);		// Rotate The Quad On The X axis ( NEW )	

	glColor3f(0.5f,0.5f,1.0f);			// Set The Color To Blue One Time Only
	glBegin(GL_QUADS);				// Draw A Quad
	glVertex3f(-1.0f, 1.0f, 0.0f);			// Top Left
	glVertex3f( 1.0f, 1.0f, 0.0f);			// Top Right
	glVertex3f( 1.0f,-1.0f, 0.0f);			// Bottom Right
	glVertex3f(-1.0f,-1.0f, 0.0f);			// Bottom Left
	glEnd();					// Done Drawing The Quad

	{						// Set the window title.
		char buf[64];

		sprintf(buf, "Rotation. Frame rate: %i.", PerformanceCounter.fps());
		Display.set_window_title(buf);
	}
}

// Update the forward_move, up_move and right_move variables when a key is pressed.
bool
Lesson04::
handle_key_pressed(const SDL_keysym *keysym) {
	switch (keysym->sym) {
		default:
			return Loop::handle_key_pressed(keysym);
	}
	return true;
}

void
Lesson04::
move(const float dt) {
	m_angle_triangle += dt*60;	// Increase The Rotation Variable For The Triangle ( NEW )
	m_angle_quad -= dt*45;		// Decrease The Rotation Variable For The Quad     ( NEW )
}

