/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: lesson_05.cpp,v 1.1.1.1 2003/07/25 19:28:31 rodzilla Exp $
*/

#include <SaM3d/display.h>
#include <SaM3d/opengl.h>
#include <SaM3d/performance_counter.h>
#include "lesson_05.h"

using namespace SaM3d;

Lesson05::
Lesson05() {
	m_camera.set_fov(0.1, 200);

	m_angle_pyramid = 0;
	m_angle_cube = 0;
}

// Init OpenGL parameters for this scene.
void
Lesson05::
init() {
	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDepthFunc(GL_LESS);
	glDisable(GL_CULL_FACE);
}

// Init OpenGL parameters for this scene.
void
Lesson05::
draw() {
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear the display.

	m_camera.activate();

	glPushMatrix();					// Store The Current Modelview Matrix
	glTranslatef(-1.5f,0.0f,-6.0f);			// Move Left 1.5 Units And Into The Screen 6.0
	glRotatef(m_angle_pyramid,0.0f,1.0f,0.0f);	// Rotate The Triangle On The Y axis

	glBegin(GL_TRIANGLES);				// Start Drawing The Pyramid
	glColor3f(1.0f,0.0f,0.0f);			// Red
	glVertex3f( 0.0f, 1.0f, 0.0f);			// Top Of Triangle (Front)
	glColor3f(0.0f,1.0f,0.0f);			// Green
	glVertex3f(-1.0f,-1.0f, 1.0f);			// Left Of Triangle (Front)
	glColor3f(0.0f,0.0f,1.0f);			// Blue
	glVertex3f( 1.0f,-1.0f, 1.0f);			// Right Of Triangle (Front)
	glColor3f(1.0f,0.0f,0.0f);			// Red
	glVertex3f( 0.0f, 1.0f, 0.0f);			// Top Of Triangle (Right)
	glColor3f(0.0f,0.0f,1.0f);			// Blue
	glVertex3f( 1.0f,-1.0f, 1.0f);			// Left Of Triangle (Right)
	glColor3f(0.0f,1.0f,0.0f);			// Green
	glVertex3f( 1.0f,-1.0f, -1.0f);			// Right Of Triangle (Right)
	glColor3f(1.0f,0.0f,0.0f);			// Red
	glVertex3f( 0.0f, 1.0f, 0.0f);			// Top Of Triangle (Back)
	glColor3f(0.0f,1.0f,0.0f);			// Green
	glVertex3f( 1.0f,-1.0f, -1.0f);			// Left Of Triangle (Back)
	glColor3f(0.0f,0.0f,1.0f);			// Blue
	glVertex3f(-1.0f,-1.0f, -1.0f);			// Right Of Triangle (Back)
	glColor3f(1.0f,0.0f,0.0f);			// Red
	glVertex3f( 0.0f, 1.0f, 0.0f);			// Top Of Triangle (Left)
	glColor3f(0.0f,0.0f,1.0f);			// Blue
	glVertex3f(-1.0f,-1.0f,-1.0f);			// Left Of Triangle (Left)
	glColor3f(0.0f,1.0f,0.0f);			// Green
	glVertex3f(-1.0f,-1.0f, 1.0f);			// Right Of Triangle (Left)
	glEnd();					// Done Drawing The Pyramid

	glPopMatrix();					// Restore The Current Modelview Matrix
	glTranslatef(1.5f,0.0f,-7.0f);			// Move Right 1.5 Units And Into The Screen 7.0
	glRotatef(m_angle_cube,1.0f,1.0f,1.0f);		// Rotate The Quad On The X axis

	glBegin(GL_QUADS);				// Start Drawing The Cube
	glColor3f(0.0f,1.0f,0.0f);			// Set The Color To Green
	glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Top)
	glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Top)
	glVertex3f(-1.0f, 1.0f, 1.0f);			// Bottom Left Of The Quad (Top)
	glVertex3f( 1.0f, 1.0f, 1.0f);			// Bottom Right Of The Quad (Top)
	glColor3f(1.0f,0.5f,0.0f);			// Set The Color To Orange
	glVertex3f( 1.0f,-1.0f, 1.0f);			// Top Right Of The Quad (Bottom)
	glVertex3f(-1.0f,-1.0f, 1.0f);			// Top Left Of The Quad (Bottom)
	glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Bottom)
	glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Bottom)
	glColor3f(1.0f,0.0f,0.0f);			// Set The Color To Red
	glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Front)
	glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Front)
	glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Front)
	glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Front)
	glColor3f(1.0f,1.0f,0.0f);			// Set The Color To Yellow
	glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Back)
	glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Back)
	glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Back)
	glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Back)
	glColor3f(0.0f,0.0f,1.0f);			// Set The Color To Blue
	glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Left)
	glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Left)
	glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Left)
	glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Left)
	glColor3f(1.0f,0.0f,1.0f);			// Set The Color To Violet
	glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Right)
	glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Right)
	glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Right)
	glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Right)
	glEnd();					// Done Drawing The Cube


	{					// Set the window title.
		char buf[64];

		sprintf(buf, "3d shapes. Frame rate: %i.", PerformanceCounter.fps());
		Display.set_window_title(buf);
	}
}

// Update the forward_move, up_move and right_move variables when a key is pressed.
bool
Lesson05::
handle_key_pressed(const SDL_keysym *keysym) {
	switch (keysym->sym) {
		default:
			return Loop::handle_key_pressed(keysym);
	}
	return true;
}

void
Lesson05::
move(const float dt) {
	m_angle_pyramid += dt*60;	// Increase The Rotation Variable For The Triangle ( NEW )
	m_angle_cube -= dt*45;		// Decrease The Rotation Variable For The Quad     ( NEW )
}

