/**************************************************************************

    This file is part of SaM3d!, yet another OpenGL-based engine.
    Copyright (C) 2001 - 2003 Rodolphe Suescun

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************
    $Id: lesson_06.cpp,v 1.3 2003/07/26 07:41:40 rodzilla Exp $
*/

#include <SaM3d/display.h>
#include <SaM3d/opengl.h>
#include <SaM3d/performance_counter.h>
#include "lesson_06.h"

using namespace SaM3d;

Lesson06::
Lesson06() : m_texture(new FileTexture("textures/texture.jpg")) {
	m_camera.set_fov(0.1, 200);

	m_xrot = 0;
	m_yrot = 0;
	m_zrot = 0;
}

// Init OpenGL parameters for this scene.
void
Lesson06::
init() {
	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDepthFunc(GL_LESS);
	glDisable(GL_CULL_FACE);
}

// Init OpenGL parameters for this scene.
void
Lesson06::
draw() {
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear the display.

	m_camera.activate();

	glTranslatef(0.0f,0.0f,-5.0f);

	glRotatef(m_xrot,1.0f,0.0f,0.0f);
	glRotatef(m_yrot,0.0f,1.0f,0.0f);
	glRotatef(m_zrot,0.0f,0.0f,1.0f);

	glColor3f(1.0, 1.0, 1.0);
	glEnable(GL_TEXTURE_2D);
	m_texture.bind();

	glBegin(GL_QUADS);
	// Front Face
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
	// Back Face
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
	// Top Face
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
	// Bottom Face
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
	// Right face
	glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
	// Left Face
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
	glEnd();


	{					// Set the window title.
		char buf[64];

		sprintf(buf, "3d shapes. Frame rate: %i.", PerformanceCounter.fps());
		Display.set_window_title(buf);
	}
}

// Update the forward_move, up_move and right_move variables when a key is pressed.
bool
Lesson06::
handle_key_pressed(const SDL_keysym *keysym) {
	switch (keysym->sym) {
		default:
			return Loop::handle_key_pressed(keysym);
	}
	return true;
}

void
Lesson06::
move(const float dt) {
	m_xrot += dt*45;	// Decrease The Rotation Variable For The Quad     ( NEW )
	m_yrot += dt*60;	// Increase The Rotation Variable For The Triangle ( NEW )
	m_zrot += dt*60;	// Increase The Rotation Variable For The Triangle ( NEW )
}

